@@ -1,8 +1,18 @@
 # (c) Nelen & Schuurmans.  GPL licensed, see LICENSE.rst.
 # -*- coding: utf-8 -*-
+"""
+Models
+++++++
+In the threedicore lines are called flow lines and represent links between
+calculation points. They can exist between 1d and 2d points, between surface
+and groundwater nodes etc. The line type is described by the kcu attribute
+For an overview of the kcu types see :ref:`kcu-label`.
+"""
+
 from __future__ import unicode_literals
 from __future__ import print_function
 
+from __future__ import absolute_import
 import numpy as np
 
 from threedigrid.admin.lines import exporters
@@ -19,6 +29,24 @@ LINE_SUBSETS = {
 
 
 class Lines(Model):
+    """
+    fields originating from threedicore:
+
+        - kcu (line type)
+        - lik ()
+        - line (index of node representing start/end point)
+
+    added fields from spatialite database:
+
+        - content_pk (primary key database)
+        - content_type (type of object as stored in
+          database, e.g v2_channel, v2_weir etc)
+
+    custom fields
+        - line_coords (coordinate pairs start/end point)
+        - line_geometries (geometries from content_type)
+
+    """
     kcu = ArrayField()
     lik = ArrayField()
     line = IndexArrayField(to='Nodes')
@@ -36,10 +64,6 @@ class Lines(Model):
             exporters.LinesOgrExporter(self),
         ]
 
-    @property
-    def v2_channels(self):
-        return self.filter(content_type='v2_channel')
-
     @property
     def pipes(self):
         return self._filter_as(Pipes, content_type='v2_pipe')
@@ -111,9 +135,9 @@ class Weirs(Lines):
         line_content_pk = self._datasource['content_pk'].value
         pk_mask = np.isin(line_content_pk, self.content_pk)
         raw_values = self._datasource['line_coords'].value[
-            :, (content_type == 'v2_weir') & pk_mask]
+            :, (content_type == b'v2_weir') & pk_mask]
 
-        return self.get_field('line_coords').get_angles_in_degrees(
+        return self._get_field('line_coords').get_angles_in_degrees(
             raw_values)
 
 
@@ -138,7 +162,6 @@ class Culverts(Lines):
 class Orifices(Lines):
     display_name = ArrayField()
     sewerage = ArrayField()
-    max_capacity = ArrayField()
     friction_type = ArrayField()
     friction_value = ArrayField()
     crest_type = ArrayField()
