@@ -1,30 +1,33 @@
 # (c) Nelen & Schuurmans.  GPL licensed, see LICENSE.rst.
 # -*- coding: utf-8 -*-
+"""
+Models
+++++++
+
+The ``Nodes`` models represent the actual calculation points of the
+threedicore and derived information like the cells they lie in.
+
+For examples how to query the ``Nodes`` model see :ref:`api-label`
+
+
+"""
+
 from __future__ import unicode_literals
 from __future__ import print_function
 
+from __future__ import absolute_import
 import numpy as np
 
-from itertools import izip
-from itertools import tee
-
 from threedigrid.orm.models import Model
 from threedigrid.orm.fields import ArrayField
+from threedigrid.orm.base.fields import BooleanArrayField
 from threedigrid.orm.fields import PointArrayField
-from threedigrid.orm.fields import PolygonArrayField
-from threedigrid.orm.utils import transform_xys
-from threedigrid.admin.utils import get_smallest_uint_dtype
+from threedigrid.orm.fields import BboxArrayField
+from threedigrid.geo_utils import transform_xys
+from threedigrid.numpy_utils import get_smallest_uint_dtype
 from threedigrid.admin.nodes import exporters
 from threedigrid.admin.nodes import subsets
-
-
-def pairwise(iterable):
-    # from https://docs.python.org/2/library/
-    # itertools.html#recipes
-    "s -> (s0,s1), (s1,s2), (s2, s3), ..."
-    a, b = tee(iterable)
-    next(b, None)
-    return izip(a, b)
+from six.moves import zip
 
 
 NODE_SUBSETS = {
@@ -37,9 +40,12 @@ class Nodes(Model):
     content_pk = ArrayField()
     seq_id = ArrayField()
     coordinates = PointArrayField()
-    cell_coords = PolygonArrayField()
+    cell_coords = BboxArrayField()
     zoom_category = ArrayField()
     node_type = ArrayField()
+    is_manhole = BooleanArrayField()
+    sumax = ArrayField()
+
     SUBSETS = NODE_SUBSETS
 
     @property
@@ -48,7 +54,7 @@ class Nodes(Model):
 
     @property
     def manholes(self):
-        return self._filter_as(Manholes, content_pk__ne=0)
+        return self._filter_as(Manholes, is_manhole=True)
 
     @property
     def added_calculationnodes(self):
@@ -71,10 +77,10 @@ class Nodes(Model):
         # data['coordinates'][1] += y0
 
         # Return [[node_id, coordinate_x + x0, coordinate_y + y0]]
-        return zip(
+        return list(zip(
             data['id'].tolist(),
             data['coordinates'][0].tolist(),
-            data['coordinates'][1].tolist())
+            data['coordinates'][1].tolist()))
 
 
 class AddedCalculationNodes(Nodes):
@@ -98,6 +104,14 @@ class Manholes(ConnectionNodes):
 
 
 class Cells(Nodes):
+    """
+    Model that represents the threedicore calculation cells. ``Cells`` are
+    a sub-class of ``Nodes`` because they share the same attributes.
+    ``Cells``, however, also have a z_coordinate, the bottom level of
+    the grid cell and cell_coords, the lower left and upper right coordinates
+    of the cells extent.
+
+    """
 
     z_coordinate = ArrayField()
 
@@ -152,9 +166,17 @@ class Cells(Nodes):
 
 
 class Grid(Model):
-    """nodm, nodn and nodk have the same size as nodes and cells
-    which is why this model lives in the nodes/model module. In fact
-    they are attributes of the cell coordinates"""
+    """
+    Implemented fields:
+
+        - nodm
+        - nodn
+        - nodk
+
+    They all have the same size as nodes and cells, which is why this
+    model lives in the nodes/model module. In fact they are attributes
+    of the cell coordinates
+    """
 
     nodm = ArrayField()  #
     nodn = ArrayField()
@@ -171,7 +193,9 @@ class Grid(Model):
         get the node grid to pixel map
 
         :param dem_pixelsize: pixelsize of the geo tiff
-        dem_shape: shape of the numpy representation of the geo tiff
+        :param dem_shape: shape of the numpy representation of the geo tiff
+
+        :return: flipped array of the dem_shape that matches the geotiff
         """
 
         # Convert nod_grid to smallest uint type possible
@@ -190,10 +214,10 @@ class Grid(Model):
 
         n_slice_start = np.array(_n * _size, dtype='int')
         n_slice_end = np.array((_n * _size) + _size, dtype='int')
-        n_slice = zip(n_slice_start, n_slice_end)
+        n_slice = list(zip(n_slice_start, n_slice_end))
         m_slice_start = np.array(_m * _size, dtype='int')
         m_slice_end = np.array((_m * _size) + _size, dtype='int')
-        m_slice = zip(m_slice_start, m_slice_end)
+        m_slice = list(zip(m_slice_start, m_slice_end))
 
         for ns, ms, idx in zip(n_slice, m_slice, _ind):
             axis_x = slice(*ns)
