@@ -26,7 +26,8 @@ NODE_TYPE__IN_SUBSETS['GROUNDWATER_ALL'] = [
 ]
 
 NODE_TYPE__IN_SUBSETS['2D_ALL'] = [
-    NODE_TYPE__IN_SUBSETS['GROUNDWATER_ALL'],
+    NODE_TYPE__EQ_SUBSETS['2D_GROUNDWATER'],
+    NODE_TYPE__EQ_SUBSETS['2D_GROUNDWATER_BOUNDARIES'],
     NODE_TYPE__EQ_SUBSETS['2D_BOUNDARIES'],
-    NODE_TYPE__EQ_SUBSETS['2D_OPEN_WATER'],
+    NODE_TYPE__EQ_SUBSETS['2D_OPEN_WATER']
 ]
