@@ -3,8 +3,13 @@
 from __future__ import unicode_literals
 from __future__ import print_function
 
+from __future__ import absolute_import
 import logging
 import os
+
+import numpy as np
+import six
+
 from threedigrid.admin.h5py_datasource import H5pyGroup
 from threedigrid.admin.idmapper import IdMapper
 from threedigrid.admin import constants
@@ -39,7 +44,6 @@ def skip_prepare(h5py_file, group_name, attr_name, overwrite):
 def prepare_lines_onedee(
         h5py_file, threedi_datasource, klass, attr_name, overwrite=False):
 
-
     if skip_prepare(h5py_file, 'lines', attr_name, overwrite):
         return
 
@@ -55,7 +59,9 @@ class GridAdminH5Prepare(object):
         # model_name, revision_number, revision_hash
         # and possible other meta data..
         if extra_attrs:
-            for key, value in extra_attrs.iteritems():
+            for key, value in six.iteritems(extra_attrs):
+                if isinstance(value, six.string_types):
+                    value = np.string_(value)
                 h5py_file.attrs[key] = value
 
         # Step 3: prepare the ID mapper
@@ -79,6 +85,9 @@ class GridAdminH5Prepare(object):
 
     @staticmethod
     def prepare_levees(h5py_file, threedi_datasource, overwrite=False):
+        if not threedi_datasource.levees:
+            return
+
         if skip_prepare(h5py_file, 'levees', 'prepared', overwrite):
             return
 
@@ -194,7 +203,7 @@ class GridAdminH5Prepare(object):
         if not has_1d:
             return
 
-        if not is_prepared(h5py_file, 'lines', 'line_prepared'):
+        if not is_prepared(h5py_file, 'lines', 'lines_prepared'):
             # Line data is missing, prepare line data first.
             GridAdminH5Prepare.prepare_lines(h5py_file, threedi_datasource)
 
@@ -265,6 +274,9 @@ class GridAdminH5Export(object):
         self.export_lines()
         self.export_grid()
         self.export_levees()
+        self.export_2d_groundwater_lines()
+        self.export_2d_openwater_lines()
+        self.export_2d_vertical_infiltration_lines()
 
     def export_nodes(self):
         """
@@ -315,11 +327,51 @@ class GridAdminH5Export(object):
         self._export_2d_grid()
         self._export_groundwater_grid()
 
+    def export_2d_groundwater_lines(self):
+        if not (self.ga.has_groundwater and self.ga.has_2d):
+            logger.info(
+                "[*] Model {} does not have 2d groundwater lines, "
+                "skipping export 2d groundwater lines...".format(
+                    self.ga.model_name)
+            )
+            return
+        dest = os.path.join(self._dest, constants.GROUNDWATER_LINES_SHP)
+        getattr(self.ga.lines.subset(
+            constants.SUBSET_2D_GROUNDWATER), self._export_method)(dest)
+
+    def export_2d_openwater_lines(self):
+        if not self.ga.has_2d:
+            logger.info(
+                "[*] Model {} does not have 2D, "
+                "skipping export 2d open water lines...".format(
+                    self.ga.model_name)
+            )
+            return
+        dest = os.path.join(self._dest, constants.OPEN_WATER_LINES_SHP)
+        getattr(self.ga.lines.subset(
+            constants.SUBSET_2D_OPEN_WATER), self._export_method)(dest)
+
+    def export_2d_vertical_infiltration_lines(self):
+        if not self.ga.has_2d:
+            logger.info(
+                "[*] Model {} does not have 2D, "
+                "skipping export 2d vertical infiltration lines...".format(
+                    self.ga.model_name)
+            )
+            return
+        dest = os.path.join(
+            self._dest, constants.VERTICAL_INFILTRATION_LINES_SHP)
+        getattr(
+            self.ga.lines.subset(constants.SUBSET_2D_VERTICAL_INFILTRATION),
+            self._export_method)(dest)
+
     def _export_groundwater_grid(self):
         if not self.ga.has_groundwater:
             logger.info(
                 "[*] Model {} does not have groundwater, "
-                "skipping export groundwater cells...".format(self.ga.model_name)
+                "skipping export groundwater cells...".format(
+                    self.ga.model_name
+                )
             )
             return
         dest_gw = os.path.join(self._dest, constants.GROUNDWATER_SHP)
