@@ -3,7 +3,8 @@
 from __future__ import unicode_literals
 from __future__ import print_function
 
-from constants import SHP_DRIVER_NAME, GEO_PACKAGE_DRIVER_NAME
+from __future__ import absolute_import
+from .constants import SHP_DRIVER_NAME, GEO_PACKAGE_DRIVER_NAME
 
 from threedigrid.orm.base.models import Model as BaseModel
 from threedigrid.orm.fields import LineArrayField
@@ -20,8 +21,8 @@ class Model(BaseModel):
                  transformed to centroids
         """
         selection = self.__do_filter()
-        for field_name in self.fields:
-            field = self.get_field(field_name)
+        for field_name in self._field_names:
+            field = self._get_field(field_name)
             if isinstance(field, LineArrayField) and field_name in selection:
                 # Use the to_centroid function of the GeomArrayField
                 # to allow different geometry to be "centroided"
@@ -30,14 +31,17 @@ class Model(BaseModel):
                         selection[field_name])
         return selection
 
+    def _is_coords(self, field_name):
+        return isinstance(self._get_field(field_name), GeomArrayField)
+
     def _includes_coords(self, selection):
         """
         Returns: true if selection contains any data that
                  has coordinates.
         """
         found = False
-        for field_name in self.fields:
-            if isinstance(self.get_field(field_name), GeomArrayField):
+        for field_name in self._field_names:
+            if isinstance(self._get_field(field_name), GeomArrayField):
                 found = True
                 break
         return found
@@ -49,10 +53,22 @@ class Model(BaseModel):
         Note: the last reproject_to is used:
             reproject_to('28992').reproject_to('4326') == reproject_to('4326')
         """
+        new_class_kwargs = dict(self.class_kwargs)
+        new_class_kwargs.update({
+            'reproject_to_epsg': target_epsg_code})
+
         return self.__init_class(
-            self.__class__, self.slice_filters,
-            only_fields=self.only_fields,
-            reproject_to_epsg=target_epsg_code)
+            self.__class__, **new_class_kwargs)
+
+    def __do_reproject_value(self, value, field_name, target_epsg_code):
+        if target_epsg_code == self.epsg_code:
+            # Already done
+            return value
+
+        field = self._get_field(field_name)
+
+        return field.reproject(
+            value, self.epsg_code, target_epsg_code)
 
     def __do_reproject(self, selection, target_epsg_code):
         """
@@ -64,8 +80,8 @@ class Model(BaseModel):
             return selection
 
         # Reproject all GeomDataField's
-        for field_name in self.fields:
-            field = self.get_field(field_name)
+        for field_name in self._field_names:
+            field = self._get_field(field_name)
             if isinstance(field, GeomArrayField) and field_name in selection:
                 # Use the reproject function of the GeomDataField
                 # to allow different geometry to be reprojected
